from cloudinit import log as logging
from cloudinit.settings import PER_INSTANCE
from cloudinit import templater
from cloudinit import util
from copy import deepcopy

from distutils.spawn import find_executable

import struct
import socket

LOG = logging.getLogger(__name__)

frequency = PER_INSTANCE


def cidr(prefix):
    return socket.inet_ntoa(struct.pack(">I", (0xffffffff << (32 - prefix)) & 0xffffffff))

def del_line(line, fn):
    try:
        contents = util.load_file(fn).split('\n')
    except:
        contents = []

    out = []
    for l in contents:
        if not l.strip().startswith(line):
            out.append(l)
    util.write_file(fn, '\n'.join(out))


def append_line(line, fn):
    try:
        content = util.load_file(fn)
    except:
        content = ''
    if line not in content:
        content += '\n' + line.strip() + '\n'
    util.write_file(fn, content)


def add_route_cmd(net, mask, gw, dev):
    if find_executable('ip'):
        del_cmd = 'ip r del {net}/{mask}'.format(net=net, mask=mask)
        add_cmd = 'ip r add {net}/{mask} via {gw} dev {dev}'.format(
            net=net, mask=mask, gw=gw, dev=dev)
    elif find_executable('route'):
        del_cmd = 'route del -net {net}/{mask}'.format(net=net, mask=mask)
        add_cmd = 'route add -net {net}/{mask} gw {gw} dev {dev}'.format(
            net=net, mask=mask, gw=gw, dev=dev)
    else:
        return ('', False)

    try:
        util.subp(del_cmd, shell=True)
    except Exception:
        pass
    return util.subp(add_cmd, shell=True)


def conf_redhat_route(name, cfg, _cloud, log, _args):
    for route_entry in cfg['route_conf']:
        (net, mask, gw, dev) = route_entry.split()
        
        try:
            add_route_cmd(net, mask, gw, dev)
        except Exception:
            pass

        if net == '0.0.0.0':
            gw_file = '/etc/sysconfig/network'
            del_line('GATEWAY=%s' % gw, gw_file)
            append_line('GATEWAY=%s' % gw, gw_file)

        gw_file = '/etc/sysconfig/network-scripts/route-{dev}'.format(dev=dev)
        del_line('{net}/{mask}'.format(net=net, mask=mask), gw_file)
        append_line('{net}/{mask} via {gw} dev {dev}'.format(net=net, mask=mask, gw=gw, dev=dev), gw_file)


def conf_debian_route(name, cfg, _cloud, log, _args):
    for route_entry in cfg['route_conf']:
        (net, mask, gw, dev) = route_entry.split()
        netmask = cidr(int(mask))

        try:
            add_route_cmd(net, mask, gw, dev)
        except Exception:
            pass

        gw_file = '/etc/network/interfaces'
        try:
            contents = util.load_file(gw_file).split('\n')
        except Exception:
            contents = []

        route_entry = 'up route add -net {net} netmask {netmask} gw {gw} dev {dev}'.format(
                    net=net, netmask=netmask, gw=gw, dev=dev)

        if 'auto {dev}'.format(dev=dev) in contents:
            i = contents.index('auto {dev}'.format(dev=dev))
            if 'up route add -net {net}'.format(net=net) in contents[i:]:
                j = contents.index('up route add -net {net}'.format(net=net))
                contents[j] = route_entry
            else:
                contents.insert(i+4, route_entry)
        else:
            contents.append( '\n'.join(('auto %s' % dev, route_entry )) )
        util.write_file(gw_file, '\n'.join(contents))



def conf_suse_route(name, cfg, _cloud, log, _args):
    for route_entry in cfg['route_conf']:
        (net, mask, gw, dev) = route_entry.split()

        try:
            add_route_cmd(net, mask, gw, dev)
        except Exception:
            pass
        
        if net == '0.0.0.0':
            gw_file = 'etc/sysconfig/network/routes'
            del_line('default', gw_file)
            append_line('default %s' % gw, gw_file)

        gw_file = '/etc/sysconfig/network/ifroute-{dev}'.format(dev=dev)
        del_line('{net}'.format(net=net), gw_file)
        append_line('{net} {gw} {mask} {dev}'.format(net=net, gw=gw, mask=mask, dev=dev), gw_file)



def handle(name, cfg, _cloud, log, _args):
    if not cfg.get('route_conf'):
        log.debug(("Skipping module named %s,"
                   " no 'route_conf' key in configuration"), name)
        return

    _distro_name = _cloud.distro.name
    if _distro_name in ('rhel', 'centos'):
        functor = conf_redhat_route
    elif _distro_name in ('ubuntu', 'debian'):
        functor = conf_debian_route
    elif _distro_name in ('sles','opensuse', ):
        functor = conf_suse_route
    elif _distro_name in ('gentoo', ):
        functor = conf_gentoo_route
    else:
        log.debug("not support distribution %s" % _distro_name)
        return
    functor(name, cfg, _cloud, log, _args)
