# vi: ts=4 expandtab
#
#    Copyright (C) 2012 Canonical Ltd.
#    Copyright (C) 2012 Hewlett-Packard Development Company, L.P.
#    Copyright (C) 2012 Yahoo! Inc.
#
#    Author: Scott Moser <scott.moser@canonical.com>
#    Author: Juerg Haefliger <juerg.haefliger@hp.com>
#    Author: Joshua Harlow <harlowja@yahoo-inc.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3, as
#    published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import re

from cloudinit.distros import rhel, get_lsb_release
from cloudinit import log as logging
from cloudinit import util

LOG = logging.getLogger(__name__)


class Distro(rhel.Distro):

    def __init__(self, name, cfg, path):
        rhel.Distro.__init__(self, name, cfg, path)
        self.cname = 'CentOS'
        self.rpmgpgkey_path = None


    def get_fn_release(self):
        reg = re.compile('%s (Linux )?release (\d+.\d+).*' % self.cname)
        check_files = ('/etc/issue',
                       '/etc/issue.net',
                       '/etc/centos-release',
                       '/etc/redhat-release',
                       '/etc/system-release')
        for fn in check_files:
            if os.path.exists(fn):
                out = reg.findall(util.load_file(fn).strip())
                if out: return out[0][-1]
        return ''
