from cloudinit import aliyun_utils
from cloudinit import util
from . import helpers

import os
import pdb
import tempfile
import httpretty as hp
import tarfile


class TestAliyunUtils(helpers.FilesystemMockingTestCase):
    def setUp(self):
        super(TestAliyunUtils, self).setUp()
        self.tmp = self.makeDir(prefix="unittest_") 
        self.patchUtils(self.tmp)
        self.patchOS(self.tmp)


    @hp.activate
    def test_do_download(self, url=None):
        if not url:
            url = 'http://192.168.112.128/cloud-init-0.7.6a1.tgz'
        hp.register_uri(hp.GET, url)
        tempdir = self.tmp
        with aliyun_utils.go2dir( tempdir ):
            target_path = '/'.join((tempdir, url.split('/')[-1]))
            aliyun_utils.do_download(url, target_path)
            self.assertTrue( os.path.exists(target_path) )

            util.del_file(target_path)
            try:
                aliyun_utils.do_download('', target_path)
            except:
                pass
            self.assertFalse( os.path.exists(target_path) )


    @hp.activate
    def test_download_package(self):
        pkg_info = {
            'download_url': 'http://192.168.112.128/cloud-init-0.7.6a1.tgz',
            'md5': 'd41d8cd98f00b204e9800998ecf8427e'
        }
        hp.register_uri(hp.GET, pkg_info['download_url'], body='')  # emty file
        work_dir = self.tmp

        status_code, desc, download_duration, pkg_path = aliyun_utils.download_package(pkg_info, work_dir)
        self.assertTrue( aliyun_utils.check_md5(pkg_info['md5'], pkg_path) )
        self.assertTrue( status_code == 0 )

        pkg_info = {
            'download_url': 'http://xxxx.tgz',
            'md5': 'xxxxxxxxxxxxx'
        }
        status_code, _, _, _ = aliyun_utils.download_package(pkg_info, work_dir)
        self.assertTrue( status_code == 40001 )

        pkg_info = {
            'download_url': 'http://192.168.112.128/cloud-init-0.7.6a1.tgz',
            'md5': 'xxxxxxxxxxxxx'
        }
        status_code, _, _, _ = aliyun_utils.download_package(pkg_info, work_dir)
        self.assertFalse( aliyun_utils.check_md5(pkg_info['md5'], pkg_path) )
        self.assertTrue( status_code == 40002 )


    def test_check_virt_what(self):
        virt_what = aliyun_utils.check_virt_what()
        self.assertTrue( virt_what )


    def test_check_md5(self):
        check_file = os.path.join(self.tmp, 'check_md5')
        open(check_file, 'w').close() # empty file
        self.assertTrue(  aliyun_utils.check_md5('d41d8cd98f00b204e9800998ecf8427e', check_file) )
        self.assertFalse( aliyun_utils.check_md5('', '') )
        self.assertFalse( aliyun_utils.check_md5('d41d8cd98f00b204e9800998ecf8427e', '') )
        self.assertFalse( aliyun_utils.check_md5('d41d8cd98f00b204e9800998', check_file) )


    def test_check_extract_pkg(self):
        tgz_file = os.path.join(self.tmp, 'check_extract.tgz')
        test_file_1 = os.path.join(self.tmp, 'test_file_1')
        test_file_2 = os.path.join(self.tmp, 'test_file_2')
        open(test_file_1, 'w').close()
        open(test_file_2, 'w').close()

        tf = tarfile.open(tgz_file, 'w:gz')
        tf.add(test_file_1, recursive=False)
        tf.add(test_file_2, recursive=False)
        tf.close()

        extract_dir = os.path.join(self.tmp, 'extract_dir')
        util.ensure_dir(extract_dir)
        aliyun_utils.extract_pkg(tgz_file, extract_dir)

        self.assertTrue( os.path.exists('extract_dir' + test_file_1) )
        self.assertTrue( os.path.exists('extract_dir' + test_file_2) )
