
from cloudinit.distros import centos, opensuse, sles
from cloudinit import util

from .. import helpers


class TestRelease(helpers.FilesystemMockingTestCase):
    def setUp(self):
        self.tmp = self.makeDir(prefix="unittest_")
        self.patchUtils(self.tmp)
        self.patchOS(self.tmp)


    def do_test_release_good(self, distro, cases):
        d = distro
        for fn, content, mj, mn in cases:
            util.write_file(fn, content)

            self.assertTrue( d.release )
            self.assertTrue( d.major_version == mj )
            self.assertTrue( d.minor_version == mn )

            util.del_file(fn)
            d._release = None


    def do_test_release_bad(self, distro, cases):
        d = distro
        for fn, content, _, _ in cases:
            util.write_file(fn, content)

            self.assertFalse( d.release )
            self.assertFalse( d.major_version )
            self.assertFalse( d.minor_version )

            util.del_file(fn)
            d._release = None


    def test_centos_good(self):
        cases = (
            ('/etc/redhat-release', 'CentOS release 5.10 (Final)', '5', '10'),
            ('/etc/issue', 'CentOS release 6.5 (Final)\nKernel \r on an \m', '6', '5'), 
            ('/etc/system-release', 'CentOS release 6.3 (Final)', '6', '3'),
            ('/etc/centos-release', 'CentOS Linux release 7.0.1406 (Core)', '7', '0' ),
        )
        return self.do_test_release_good(centos.Distro('centos', {}, {}), cases)


    def test_centos_bad(self):
        cases = (
            ('/etc/issue', '', '', ''),
            ('/etc/issue', '\S\\nKernel \r on an \m', '7', '0' ),
            ('/etc/system-release', 'CentOS release 6 (Final)', '6', '3'),
        )
        return self.do_test_release_bad(centos.Distro('centos', {}, {}), cases)


    def test_opensuse_good(self):
        suse_release = """\
openSUSE 13.1 (x86_64)
VERSION = 13.1
CODENAME = Bottle
# /etc/SuSE-release is deprecated and will be removed in the future, use /etc/os-release instead
"""
        os_release = """\
NAME=openSUSE
VERSION="13.1 (Bottle)"
VERSION_ID="13.1"
PRETTY_NAME="openSUSE 13.1 (Bottle) (x86_64)"
ID=opensuse
ANSI_COLOR="0;32"
CPE_NAME="cpe:/o:opensuse:opensuse:13.1"
BUG_REPORT_URL="https://bugs.opensuse.org"
HOME_URL="https://opensuse.org/"
ID_LIKE="suse"
"""
        cases = (
            ('/etc/issue', 'Welcome to openSUSE 13.1 "Bottle" - Kernel', '13', '1'),
            ('/etc/SuSE-release',  suse_release, '13', '1'),
            ('/etc/os-release', os_release, '13', '1'),
        )
        return self.do_test_release_good(opensuse.Distro('opensuse', {}, {}), cases)

    def test_opensuse_bad(self):
        cases = (
            ('/etc/issue', '', '13', '1'),
            ('/etc/SuSE-release',  '', '13', '1'),
            ('/etc/os-release', '', '13', '1'),
        )
        return self.do_test_release_bad(opensuse.Distro('opensuse', {}, {}), cases)

    def test_sles_good(self):

        suse_release = """\
SUSE Linux Enterprise Server 11 (x86_64)
VERSION = 11
PATCHLEVEL = 2
"""
        cases = (
            ('/etc/issue', r'Welcome to SUSE Linux Enterprise Server 11 SP3 (x86_64) - Kernel (\l).', '11', '3'),
            ('/etc/SuSE-release', suse_release, '11', '2')
        )
        return self.do_test_release_good(sles.Distro('sles', {}, {}), cases)

    def test_sles_bad(self):
        cases = (
            ('/etc/issue', '', '11', '3'),
            ('/etc/SuSE-release', '', '11', '2')
        )
        return self.do_test_release_bad(sles.Distro('sles', {}, {}), cases)
