
import os
import logging
import cloudinit.cloud
import glob

from cloudinit import util
from cloudinit.config import cc_ip_segment
from cloudinit.distros import centos
from cloudinit.sources import DataSourceAliYun
from cloudinit.distros import centos, ubuntu, debian, sles, opensuse

from .. import helpers

import pdb

LOG = logging.getLogger(__name__)

class TestIPsegment(helpers.FilesystemMockingTestCase):
    def setUp(self):
        super(TestIPsegment, self).setUp()
        self.tmp = self.makeDir(prefix="unittest_")

        ip, mask = '100.64.100.64', 29
        self.cfg = { 'ip_segment_conf': '%s %s' % (ip, mask) }
        self.ip_list = cc_ip_segment.calc_ip_segment(ip, mask)

        self.paths = self.getCloudPaths()
        self.patchUtils(self.tmp)
        self.patchOS(self.tmp)


    def test_redhat_ip_segment(self):
        d = centos.Distro('centos', self.cfg, self.paths)
        cloud = cloudinit.cloud.Cloud(datasource='', paths=self.paths, cfg='', distro=d, runners='')

        cc_ip_segment.handle('ip_segment', self.cfg, cloud, LOG, [])

        ip_list = []
        for ip_file in glob.glob('%s/etc/sysconfig/network-scripts/ifcfg-lo:*' % self.tmp):
            with open(ip_file) as f:
                ip_line = filter( lambda x: x.startswith('IPADDR='), f.readlines() )
                if not ip_line: continue
                ip_list.append( ip_line[0].split('=')[-1].strip() )

        self.assertTrue( set(ip_list) == set(self.ip_list) )


    def test_debian_ip_segment(self):
        d = debian.Distro('ubuntu', self.cfg, self.paths)
        cloud = cloudinit.cloud.Cloud(datasource='', paths=self.paths, cfg='', distro=d, runners='')

        cc_ip_segment.handle('ip_segment', self.cfg, cloud, LOG, [])

        ip_list = []
        ifn = self.tmp + '/etc/network/interfaces'
        ifn_contents = '\n'.join( [i.strip() for i in util.load_file(ifn).split('\n') if not i.strip().startswith('#')] )

        for i in ifn_contents.split('auto'):
            if i.strip().startswith('lo:'):
                for l in i.split('\n'):
                    if l.startswith('address'):
                        ip_list.append( l.split()[-1].strip() )

        self.assertTrue( set(ip_list) == set(self.ip_list) )


    def test_suse_ip_segment(self):
        d = sles.Distro('sles', self.cfg, self.paths)
        cloud = cloudinit.cloud.Cloud(datasource='', paths=self.paths, cfg='', distro=d, runners='')

        cc_ip_segment.handle('ip_segment', self.cfg, cloud, LOG, [])

        ip_list = []
        for ip_file in glob.glob('%s/etc/sysconfig/network/ifcfg-lo:*' % self.tmp):
            with open(ip_file) as f:
                ip_line = filter( lambda x: x.startswith('IPADDR_'), f.readlines() )
                if not ip_line: continue
                ip_list.append( ip_line[0].split('=')[-1].strip() )

        self.assertTrue( set(ip_list) == set(self.ip_list) )


    def test_gentoo_ip_segment(self):
        pass
