import os
import logging
import shutil
import cloudinit.cloud

from cloudinit import util
from cloudinit.config import cc_ntp_conf
from cloudinit.sources import DataSourceAliYun

from .. import helpers

LOG = logging.getLogger(__name__)

class TestNtp(helpers.FilesystemMockingTestCase):
    def setUp(self):
        super(TestNtp, self).setUp()
        self.tmp = self.makeDir(prefix="unittest_")

        template_dir = self.resourceLocation()
        shutil.copytree(template_dir, os.path.join(self.tmp, template_dir))
        self.patchUtils(self.tmp)

    def test_ntp_conf(self):
        cfg = {
            'ntp-conf': {
                'ntp-servers':
                    ["ntp1.aliyun.com",
                     "ntp2.aliyun.com",
                     "ntp3.aliyun.com",
                     "ntp4.aliyun.com",
                     "ntp5.aliyun.com",
                     "ntp6.aliyun.com",
                     "ntp1.cloud.aliyuncs.com",
                     "ntp2.cloud.aliyuncs.com",
                     "ntp3.cloud.aliyuncs.com",
                     "ntp4.cloud.aliyuncs.com",
                     "ntp5.cloud.aliyuncs.com",
                     "ntp6.cloud.aliyuncs.com",
                     "ntp7.cloud.aliyuncs.com",
                     "ntp8.cloud.aliyuncs.com",
                     "ntp9.cloud.aliyuncs.com",
                     "ntp10.cloud.aliyuncs.com",
                     "ntp11.cloud.aliyuncs.com",
                     "ntp12.cloud.aliyuncs.com"]
            }
        }

        paths = self.getCloudPaths()
        ds = DataSourceAliYun.DataSourceAliYun(cfg, '', paths)
        ds.metadata = cfg
        cloud = cloudinit.cloud.Cloud(datasource=ds, paths=paths, cfg='', distro='', runners='')

        cc_ntp_conf.handle('ntp_conf', cfg, cloud, LOG, [])

        ntp_list = []
        with open(self.tmp + '/etc/ntp.conf') as f:
            for i in f.readlines():
                if i.strip().startswith('server'):
                    ntp_list.append( i.strip().split()[1] )

        self.assertTrue( set(cfg['ntp-conf']['ntp-servers']).issubset(set(ntp_list))  )
