import os
import logging
import shutil

import cloudinit.cloud
from cloudinit import util
from cloudinit.config import cc_resolv_conf

from .. import helpers

LOG = logging.getLogger(__name__)

class TestResolv(helpers.FilesystemMockingTestCase):
    def setUp(self):
        super(TestResolv, self).setUp()
        self.tmp = self.makeDir(prefix="unittest_")

        template_dir = self.resourceLocation()
        shutil.copytree(template_dir, os.path.join(self.tmp, template_dir))

        self.patchUtils(self.tmp)
        #self.patchOS(self.tmp)

    def test_resolv_conf(self):
        cfg = {
            'manage_resolv_conf': 'True',
            'resolv_conf': {
                    'options': {
                        'attempts': '1',
                        'timeout': '1',
                        'rodate': True
                },
                    'nameservers': [
                        '223.5.5.5',
                        '223.6.6.6'
                    ]
            }
        }

        paths = self.getCloudPaths()
        cloud = cloudinit.cloud.Cloud(datasource='', paths=paths, cfg='', distro='', runners='')

        cc_resolv_conf.handle('resolv_conf', cfg, cloud, LOG, [])

        nameserver_list = []
        with open(self.tmp + '/etc/resolv.conf') as f:
            for i in f.readlines():
                if i.strip().startswith('nameserver'):
                    nameserver_list.append( i.strip().split()[1] )

        self.assertTrue( set(cfg['resolv_conf']['nameservers']) == (set(nameserver_list))  )
