
import os
import logging

import cloudinit.cloud

from cloudinit import util
from cloudinit.config import cc_source_address
from cloudinit.sources import DataSourceAliYun
from cloudinit.distros import centos, ubuntu, debian, sles, opensuse

from .. import helpers

import pdb

LOG = logging.getLogger(__name__)


class TestSourceAddress(helpers.FilesystemMockingTestCase):
    def setUp(self):
        super(TestSourceAddress, self).setUp()
        self.tmp = self.makeDir(prefix="unittest_")
        self.cfg = { 'source-address': ['http://mirrors.aliyun.com', 'http://mirros.aliyuncs.com'] }
        self.paths = self.getCloudPaths()

        import shutil
        template_dir = self.resourceLocation()
        shutil.copytree(template_dir, os.path.join(self.tmp, template_dir))

        self.patchUtils(self.tmp)
        self.patchOS(self.tmp)

    def test_source_address(self):
        pass

    def test_yum_source_address(self):
        d = centos.Distro('centos', self.cfg, self.paths)
        d._release = '6.5'
        ds = DataSourceAliYun.DataSourceAliYun(self.cfg, '', self.paths)
        ds.metadata = self.cfg
        cloud = cloudinit.cloud.Cloud(datasource=ds, paths=self.paths, cfg='', distro=d, runners='')
        cc_source_address.handle('source_address', self.cfg, cloud, LOG, [])

        self.assertTrue(os.path.exists("/etc/yum.repos.d/CentOS-Base-Aliyun.repo") )
        self.assertTrue(os.path.exists("/etc/yum.repos.d/epel-Aliyun.repo") )

    def test_apt_source_address(self):
        d = debian.Distro('ubuntu', self.cfg, self.paths)
        ds = DataSourceAliYun.DataSourceAliYun(self.cfg, '', self.paths)
        ds.metadata = self.cfg
        cloud = cloudinit.cloud.Cloud(datasource=ds, paths=self.paths, cfg='', distro=d, runners='')
        cc_source_address.handle('source_address', self.cfg, cloud, LOG, [])

        self.assertTrue( os.path.exists("/etc/apt/sources.list.d/sources-aliyun-0.list") )

    def test_sles_source_address(self):
        d = sles.Distro('sles', self.cfg, self.paths)
        d._release = '11.3'
        ds = DataSourceAliYun.DataSourceAliYun(self.cfg, '', self.paths)
        ds.metadata = self.cfg
        cloud = cloudinit.cloud.Cloud(datasource=ds, paths=self.paths, cfg='', distro=d, runners='')
        cc_source_address.handle('source_address', self.cfg, cloud, LOG, [])

        self.assertTrue( os.path.exists("/etc/zypp/repos.d/SLES{release}-SP{sp_version}-0.repo".format(
            release=d.major_version, sp_version=d.sp_version)) )

    def test_opensuse_source_address(self):
        d = opensuse.Distro('opensuse', self.cfg, self.paths)
        d._release = '13.1'
        ds = DataSourceAliYun.DataSourceAliYun(self.cfg, '', self.paths)
        ds.metadata = self.cfg
        cloud = cloudinit.cloud.Cloud(datasource=ds, paths=self.paths, cfg='', distro=d, runners='')
        cc_source_address.handle('source_address', self.cfg, cloud, LOG, [])

        self.assertTrue( os.path.exists("/etc/zypp/repos.d/openSUSE-%s-Oss-0.repo" % d.release) )
